; programme to use pic 16f88 as i2c slave 

	#include p16f88.inc
	 list p=p16f88
;
;	
	
uSec10 equ h'20'		;delay variables - always included just in case
ms1 equ h'21'
sec1 equ h'22'
ms10 equ h'23'


delay_byte equ h'24'
databyte equ h'25'		;register for data to o/p or i/p
buff_size equ 8		;number of internal addresses
offset equ h'26'		;internal address pointer
addr0 equ h'27'		;internal memory locations
addr1 equ h'28'		;more or less as required or ram permits
addr2 equ h'29'
addr3 equ h'2a'		;eight should be plenty
addr4 equ h'2b'
addr5 equ h'2c'
addr6 equ h'2d'
addr7 equ h'2e'

i2cdata equ h'2f'		;display variables

;//// put slave variables here	




tempw equ h'70'		;interrupt "stack"
status_temp equ h'71'	;independent of banks
fsr_temp equ h'72'


address equ b'00001000'	;i2c device base address /// change to suit your needs

sda equ 1		;i2c pins
scl equ 4

	
	

	__config  config1, _lvp_off & _boden_off &  _mclr_off &  _wdt_off &  _intrc_io	;& _cp_all


	org 0
	goto start
	org 4
	goto interrupt
	org 6

start:
	clrf porta		;get rid of any leftover data
	clrf portb
	bcf status,irp
	bcf status,rp1
	bsf status,rp0		;bank 1
	clrf ansel
	movlw b'11110000'
	movwf trisa		;port a inputs and outputs
	movlw b'11111111'
	movwf trisb		;port b inputs
	bsf trisb,1		;scl and sda as inputs for i2c
	bsf trisb,4
	movlw b'01100000'	;set int rc osc to 4MHz
	movwf osccon
	clrf sspstat		;clear ssp status register
	bcf status,rp0		;select bank 0
	movf portb,w
	andlw b'00001100'	;mask for address bits
	movwf tempw		;save bits in temp register
	bcf status,c		;clear carry bit
	rrf tempw,w		;shit bits to pos 1 and 2
	iorlw address		;combine with base address
	bsf status,rp0		;bank 1
	movwf sspadd		;set slave address
	bsf pie1,sspie		;enable interrupts
	bcf status,rp0		;bank 0
	bcf pir1,sspif
	movlw b'00110110'	;set up for slave read no interrupt
	movwf sspcon
	bsf intcon,peie		
	bsf sspcon,sspen
	bsf intcon,gie

;===================================================================
;                    end of pic setup routine
;  		beginning of main-loop initialisation
;===================================================================
	clrf offset		;set up indirect address for slave regs


;===================================================================
;===================================================================
;       main slave function programme starts here
;===================================================================
;===================================================================
	clrf addr0	
	clrf addr1
	clrf i2cdata


main_loop:
	movf addr1,f
	btfss status,z		;i.e addr1 = 0
	goto main_loop
	clrf addr0
	clrf addr1

;//// main part of slave gets written here//////

	goto main_loop

;===================================================================
;	start of main_loop routines etc.
;===================================================================


;===================================================================
;delay routines - set for 4MHz clock speed
;===================================================================


delay10:			;10uSec per loop
	movwf usec10		;come in with uSx10 count in w reg
delay10a:
	nop			;pad out to 10uSecs
	nop
	nop
	nop
	nop
	nop
	decfsz usec10,f
	goto delay10a		;loop count 10uSec loops
	return

delayms:
	movwf ms1		;come in with mSec count in W reg
delaymsa:
	movlw 100		;set up for 100 x 10uSec = 1 mSec
	call delay10
	decfsz ms1,f
	goto delaymsa
	return

delayms10:			;come in with mSx10 count in W reg
	movwf ms10
delayms10a:
	movlw 10
	call delayms
	decfsz ms10,f
	goto delayms10a
	return

delaysec:
	movwf sec1		;come in with seconds count in W reg
delayseca:
	movlw 100		;1000 mSec
	call delayms10			
	decfsz sec1,f
	goto delayseca
	return

;========================================================================
;	End of Main Loop Routines etc.
;	Beginning of interrupt routines
;========================================================================
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;========================================================================
;	this is the data i/o section - i2c slave main routine
;========================================================================
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

interrupt:
	movwf tempw		;save contents of w reg
	swapf status,w
	movwf status_temp	;save contents of status reg
	movf fsr,w
	movwf fsr_temp		;save fsr register
	clrf status		;bank 0
	bcf pir1,sspif

	movf sspbuf,w		;clear data buffer 
	movwf databyte		;save in databyte
	bsf status,rp0		;bank 1
	btfsc sspstat,d_a	;check if data or address
	goto data_mode

;========================================================================
;	get and process mode byte	
;========================================================================

	btfsc sspstat,r_w	;check if we need a mode byte
	goto send
	call inbyte		;read mode byte
	movlw buff_size		;get number of internal addresses
	subwf databyte,w	;check if valid address
	btfsc status,c		;valid if carry is 0 
	clrf databyte		;if bad address, assume address 0
	movf databyte,w		;good address, so put it in <offset>
	movwf offset
	goto int_end

data_mode:

	btfss sspstat,r_w	;check if read or write
	goto receive		;input data
send:
	bcf status,rp0		;bank 0
	movlw addr0		;physical location of address block	
	addwf offset,w		;point to required address
	movwf fsr		;put address in indirect address mode
	movf indf,w		;get data to be output
	btfsc sspcon,ckp	;check if another byte
	goto int_end		;no so finish
	movwf sspbuf		;send data to master
	nop			;delay needed to correct timing
	nop
	nop
	nop
	nop
	bsf sspcon,ckp
	bcf pir1,sspif
	incf offset,f
	movlw buff_size
	subwf offset,w
	btfsc status,z
	clrf offset
	goto int_end

receive:
	bcf status,rp0		;bank 0
	movlw addr0		;physical location of address block
	addwf offset,w		;point to required address
	movwf fsr		;put address in indirect address mode
	movf databyte,w
	movwf indf		;put data in appropriate address
	incf offset,f		;automatic increment of address
	movlw buff_size
	subwf offset,w		;check if exceeded address limit
	btfsc status,z
	clrf offset		;back to start of addresses
	goto int_end	

;========================================================================


int_end:

	bcf status,rp0		;bank 0
	movf fsr_temp,w
	movwf fsr
	bcf pir1,sspif		;clear the interrupt flag		
	swapf status_temp,w	;restore temps
	movwf status
	swapf tempw,f
	swapf tempw,w

	retfie  	
		

inbyte:
	bcf status,rp0		;bank 0
	btfss pir1,sspif	;wait for databyte in
	goto inbyte
	movf sspbuf,w		;read data buffer and put in databyte reg
	movwf databyte
	bcf pir1,sspif		;clear interrupt flag
	return		
	
	end			;mandatory instruction